@@ -1,4 +1,4 @@
-# $Id: Rule.pm,v 1.111 2011/06/21 20:20:45 pfeiffer Exp $
+# $Id: Rule.pm,v 1.114 2011/09/15 21:00:06 pfeiffer Exp $
 use strict qw(vars subs);
 
 package Mpp::Rule;
@@ -64,7 +64,7 @@ sub build_cwd { $_[0]{MAKEFILE}{CWD} }
 sub build_cache {
   exists $_[0]{BUILD_CACHE} ? $_[0]{BUILD_CACHE} :
   exists $_[0]{MAKEFILE}{BUILD_CACHE} ? $_[0]{MAKEFILE}{BUILD_CACHE} :
-  $Mpp::global_build_cache;
+  $Mpp::BuildCache::global;
 }
 
 #
@@ -494,7 +494,7 @@ build inaccuracies.
 =cut
 
 sub mark_scaninfo_uncacheable {
-  $_[0]{SCANINFO_UNCACHEABLE} = 1;
+  undef $_[0]{xSCANINFO_UNCACHEABLE};
 }
 
 =head2 cache_scaninfo
@@ -517,7 +517,7 @@ sub cache_scaninfo {
   my ($self, $targets) = @_;
 
   die if exists $self->{SCAN_FAILED};
-  unless($Mpp::nocache_scaninfo || $self->{SCANINFO_UNCACHEABLE}) {
+  unless($Mpp::nocache_scaninfo || exists $self->{xSCANINFO_UNCACHEABLE}) {
     # Wipe out unnecessary info from IMPLICIT_DEPS:
     while(my ($tag, $th) = each %{$self->{IMPLICIT_DEPS}}) {
       $tag = $self->{META_DEPS}{$tag};
@@ -1413,10 +1413,7 @@ package Mpp::DefaultRule;
 #
 # Establish a default rule for a new file:
 #
-sub new { bless { TARGET => $_[1] }, $_[0] }
-				# Make the rule object.	 Declare it local
-				# so it's accessible to any subroutines that
-				# Mpp::Makefile::expand_text calls. (TODO: What is local here?)
+sub new { bless { TARGET => $_[1] }}
 
 sub build_cwd { $Mpp::File::CWD_INFO }
 
@@ -1456,7 +1453,7 @@ sub execute {
 				# This is to handle things like
 				# FORCE:
 				# which are really just dummy phony targets.
-    } elsif( exists $target->{IS_PHONY} ) {
+    } elsif( exists $target->{xPHONY} ) {
       0;
     } else {
       Mpp::print_error( 'No rule to make ', $target );
@@ -1525,14 +1522,19 @@ sub build_check {
 
   return 'stale symbolic link' if exists $tinfo->{TEMP_BUILD_INFO};
 
-  warn '`' . absolute_filename( $tinfo ) . "' is a stale file generated by makepp,\n" .
-    "  but there is no rule to build it any more.  Makepp assumes that it\n" .
-    "  is now a source file.  If this is correct, then you should add it\n" .
-    "  to your source control system (if any), and touch the file to make\n" .
-    "  this warning go away.  Otherwise, you should remove the file,\n" .
-    "  because the targets that depend on it will not be reproducible.\n" .
-    "  makepp --rm-stale would automatically do this for you.\n"
-      if Mpp::File::is_stale $tinfo;
+  if( Mpp::File::is_stale $tinfo ) {
+    if( defined $Mpp::Recursive::traditional || defined $Mpp::Recursive::hybrid ) {
+      warn 'info: I hope stale `' . absolute_filename( $tinfo ) . "' was generated by a parallel \$(MAKE).\n";
+    } else {
+      warn '`' . absolute_filename( $tinfo ) . "' is a stale file generated by makepp,\n" .
+	"  but there is no rule to build it any more.  Makepp assumes that it\n" .
+	"  is now a source file.  If this is correct, then you should add it\n" .
+	"  to your source control system (if any), and touch the file to make\n" .
+	"  this warning go away.  Otherwise, you should remove the file,\n" .
+	"  because the targets that depend on it will not be reproducible.\n" .
+	"  makepp --rm-stale would automatically do this for you.\n";
+    }
+  }
 
   # It would be faster to check for existence first, but that doesn't work,
   # because then if the file were from a repository but the repository version
